import os
import logging
from kflow import extract

def _getRuntimeEnviroment() -> str:

    try:
        from airflow.models import Variable        
        try:
            env = Variable.get("ENV")
        except KeyError:
            logging.warning("Variable ENV not found, we default to local environment")
    except ModuleNotFoundError as err:
        logging.warning("Airflow not found, we default to local environment")        
        env = 'dev'

    logging.info(f"Working on '{env}' environment")

    return env

def S3Connect() -> str:

    # TODO: Esta función debería devolver la conexión directamente, utilizando boto3.
    # TODO: Documentar

    env = _getRuntimeEnviroment()

    if env == 'prod':
        from airflow.models import Variable
        try:
            aws_access_key_id = Variable.get("aws_access_key_id")
            aws_secret_access_key = Variable.get("aws_secret_access_key")
        except KeyError:
            logging.warning("Variable ENV not found, we default to local environment")
    elif env == 'dev':
        S3_conn = os.environ["S3"].strip('][').split(',')
        aws_access_key_id = S3_conn[0]
        aws_secret_access_key = S3_conn[1]
    else:
        raise ValueError('Current environment not recognized')

    return [aws_access_key_id,aws_secret_access_key]

def getHubspotApiKey(hubspot_env:str) -> str:

    """
    airflow_env:str
    Variable de ambiente para acceder a Hubspot, hay cuatro opciones:

        Para ejecutar un DAG en Airflow    
            "aws_hubspot_prod" - ambiente productivo en airflow
            "aws_hubspot_test" - ambiente de pruebas en airflow

        Para ejecutar en ambiente local
            "local_HubspotProd" - ambiente productivo en local
            "local_HubspotTest" - ambiente de pruebas en local
    
    TODO: hubspot_env solo debería determinar a qué ambiente de hubspot apuntar. 
        Si se ejecuta en Airflow o en local, depende de 'env' y _getRuntimeEnvironment.
    TODO: Esto debería devolver el cliente de Hubspot directamente (y renombrarse a getHubspotClient).
        Así abstraemos la importación de la librería de Hubspot.
    """

    env = _getRuntimeEnviroment()

    if env == 'prod':
        from airflow.models import Variable
        try:
            conn_hub = Variable.get(f"{hubspot_env}")
        except KeyError:
            logging.warning(f"Variable {hubspot_env} not found, we default to local environment")
    elif env == 'dev':
        try:
            conn = extract.LakeFileAsJson("auth/","authn_klog.json")
            conn_hub = conn[hubspot_env]
        except KeyError:
            logging.error(f"Variable {hubspot_env} not found in environment, cannot stablish connection")
            return
    else:
        raise ValueError('Current environment not recognized')
       
    return conn_hub

def getConnectionDB(base_env:str):
    
    """
    TODO: Formato correcto de documentación.
    base_env:str
    Base de datos y ambiente:

        Para ejecutar un DAG en Airflow    
            "aws_pro_warehouse" - ambiente productivo en airflow warehouse
            "aws_pro_prisma" - ambiente productivo en airflow prisma

        Para ejecutar en ambiente local
            "local_WarehouseProd" - ambiente productivo en local warehouse
            "local_PrismaPro" - ambiente productivo en airflow prisma     
    """

    env = _getRuntimeEnviroment()    

    if env == "prod":
        try:      
            from airflow.providers.postgres.hooks.postgres import PostgresHook            
            hook = PostgresHook(postgres_conn_id=base_env)  # aws_pro_warehouse / aws_pro_prisma
            connection = hook.get_connection(base_env)
            # TODO Aqui hay un problema, pues a redshift conviene conectarse con el tipo de conexión redshift+psycopg2 (en vez de postgresql)
            conn_bd = f"postgresql://{connection.login}:{connection.password}@{connection.host}:{connection.port}/{connection.schema}"
        except:
            logging.warning(f"base_env {base_env} not found")
            raise    
    elif env == 'dev':
        try:
            conn = extract.LakeFileAsJson("auth/","authn_klog.json")
            conn_bd = conn[base_env]   # local_WarehouseProd / local_PrismaPro
        except KeyError:
            logging.error(f"Variable {base_env} not found in environment, cannot stablish connection")
            raise
    else:
        raise ValueError('Current environment not recognized')

    from sqlalchemy import create_engine
    bd_engine = create_engine(conn_bd)
    
    return bd_engine

def getConnectionGoogleSheet(auth:str="SERVICE"):    # old GoogleSheetAuth
    
    """
    Devuelve el servicio de Google Sheets
    
    Parameters
    ----------
    auth: str
        Indica tipo de autenticación a usar. Tipos:
        * SERVICE: Autenticación por cuenta de servicio. Esta se basa en una service account a la
            que hay que compartirle el sheet que queramos accesar.
    """
   
    from googleapiclient.discovery import build
    from google.oauth2 import service_account

    scopes = ['https://www.googleapis.com/auth/spreadsheets']
    if auth == "SERVICE":
        service_account_json = extract.LakeFileAsJson("auth/","data-and-bi-342313-9363b0ec39cf.json")
        credentials = service_account.Credentials.from_service_account_info(service_account_json, scopes=scopes)
        return build('sheets', 'v4', credentials=credentials)
    else:
        logging.error("Unidentified auth")
        return None